﻿#region MIT License
/*

Copyright (C) 2011 by Timothy A. Barela

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/
#endregion

using System;
using System.Diagnostics;
using System.Collections.Generic;
using System.IO;
using System.Reflection;
using Mono.Cecil;

namespace Sharpener
{
	/// <summary>
	/// Runs a collection of Passes on a Mono.Cecil.AssemblyDefinition.
	/// </summary>
	public class AssemblyOptimizer
	{
		public TextWriter TextWriter { get; set; }
		public List<Pass> Passes { get; protected set; }

		public AssemblyOptimizer()
		{
			TextWriter = Console.Out;
			Passes = new List<Pass>();
		}

		public AssemblyOptimizer( params Pass[] passes ) : this(passes as IEnumerable<Pass>) { }

		public AssemblyOptimizer( IEnumerable<Pass> passes )
		{
			TextWriter = Console.Out;
			Passes = new List<Pass>(passes);
		}

		public bool Optimize( AssemblyDefinition assembly )
		{
			int modified = 0;
			int passNum = 1;

			foreach(Pass pass in Passes)
			{
				TextWriter.WriteLine("\n--- {0} ---", pass.StartingMessage);
				modified += pass.Run(assembly) ? 1 : 0;
				TextWriter.WriteLine("* {0}", pass.ClosingMessage);
				passNum++;
			}

			TextWriter.WriteLine();

			return modified > 0;
		}
	}
}