﻿#region MIT License
/*

Copyright (C) 2011 by Timothy A. Barela

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/
#endregion

using System;
using System.Collections.Generic;
using System.Diagnostics;
using Mono.Cecil;
using Mono.Cecil.Cil;
using Mono.Cecil.Rocks;
using Sharpener.Analysis;

namespace Sharpener
{
	internal class DevelopmentPass : Pass
	{
		public override bool Run( AssemblyDefinition assembly )
		{
			//testBasicBlocks(assembly);
			testMethodControlFlowGraph(assembly);
			return false;
		}

		private void testBasicBlocks( AssemblyDefinition assembly )
		{
			Debug.Print("Testing BasicBlock in assembly entry point.");

			var main = assembly.MainModule.EntryPoint;

			var blocks = BasicBlock.ScanForBlocks(main.Body.Instructions);

			int index = 0;

			foreach(var b in blocks)
			{
				Debug.Print("BasicBlock #{0}", index);
				Debug.Print("Head:{0}", b.Head);
				Debug.Print("Tail:{0}", b.Tail);
				index++;
			}
			
		}

		private void testMethodControlFlowGraph( AssemblyDefinition assembly )
		{
			Debug.Print("Testing MethodControlFlowGraph in assembly entry point.");

			var main = assembly.MainModule.EntryPoint;

			var cfg = new MethodControlFlowGraph(main);

			var b = cfg.EntryBlock;

			//Debug.Print("EntryBlock");
			//Debug.Print("Head:{0}", b.Head);
			//Debug.Print("Tail:{0}", b.Tail);

			Debug.Print(cfg.EntryBlock.ToString());
		}
	}
}