﻿#region MIT License
/*

Copyright (C) 2011 by Timothy A. Barela

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/
#endregion

using System;
using System.Collections.Generic;
using System.Diagnostics;
using Mono.Cecil;
using Mono.Cecil.Cil;
using Mono.Cecil.Rocks;

namespace Sharpener
{
	internal class ParameterMapping : Dictionary<ParameterReference,VariableReference>
	{
		static Dictionary<OpCode,OpCode> opcodeRemapping;

		public MethodDefinition SourceMethod { get; private set; }
		
		static ParameterMapping()
		{
			opcodeRemapping = new Dictionary<OpCode,OpCode>()
			{
				{OpCodes.Ldarg,OpCodes.Ldloc},
				{OpCodes.Starg,OpCodes.Stloc},
				{OpCodes.Ldarga,OpCodes.Ldloc}
			};
		}

		internal ParameterMapping(MethodDefinition srcMethod,MethodDefinition dstMethod)
		{
			SourceMethod = srcMethod;

			foreach(var param in srcMethod.Parameters)
			{
				var localVar = new VariableDefinition(param.ParameterType);
				dstMethod.Body.Variables.Add(localVar);
				Add(param,localVar);
			}
		}

		internal void Remap(IList<Instruction> instructions)
		{
			foreach(var inst in instructions)
			{
				OpCode result;

				if(opcodeRemapping.TryGetValue(inst.OpCode,out result))
				{
					inst.OpCode = result;
					inst.Operand = this[inst.Operand as ParameterReference];
				}
				//else
				//{
				//    throw new NotImplementedException(string.Format("The arg type instruction '{0}' " +
				//                                                    "is not currently handled.",
				//                                                    inst.OpCode.Name));
				//}
			}
		}

		private bool isArgInstruction(Instruction inst)
		{
			return (inst.OpCode.OperandType == OperandType.InlineArg) ||
					(inst.OpCode.OperandType == OperandType.ShortInlineArg);
		}
	}
}