﻿#region MIT License
/*

Copyright (C) 2011 by Timothy A. Barela

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/
#endregion

using System;
using System.Reflection;
using Mono.Cecil;

namespace Sharpener
{
	/// <summary>
	/// Removes the reference to the Sharpener Assembly.
	/// </summary>
	public class RemoveSharpenerReferencePass : Pass
	{
		bool removed;

		public override string StartingMessage
		{
			get
			{
				return "Removing reference to sharpener assembly.";
			}
		}

		public override string ClosingMessage
		{
			get
			{
				if(removed)
					return "Reference removed.";
				else
					return "No reference was found.";
			}
		}

		public override bool Run( AssemblyDefinition assembly )
		{
			var sharpenerAssemblyFullName = Assembly.GetExecutingAssembly().FullName;

			for(int i = 0; i < assembly.MainModule.AssemblyReferences.Count; i++)
			{
				var assref = assembly.MainModule.AssemblyReferences[i];

				if(assref.FullName == sharpenerAssemblyFullName)
				{
					assembly.MainModule.AssemblyReferences.RemoveAt(i);
					return (removed=true);
				}
			}
			
			return (removed=false);
		}
	}
}