﻿#region MIT License
/*

Copyright (C) 2011 by Timothy A. Barela

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/
#endregion

using System;
using System.Diagnostics;
using Mono.Cecil;
using Mono.Cecil.Cil;
using Mono.Cecil.Metadata;

namespace Sharpener
{
	public static class Utility
	{
		public static string StringifyMethod( MethodDefinition method )
		{
			var sb = new System.Text.StringBuilder(1024);

			sb.AppendFormat("//code size: {0}\n", method.Body.CodeSize);
			sb.AppendFormat(".method {0}\n", method.FullName);
			sb.Append("{\n");
			sb.AppendFormat("\t.maxstack {0}\n", method.Body.MaxStackSize);

			if(method.Body.HasVariables)
			{
				sb.AppendFormat("\t.locals {0} \n", method.Body.InitLocals ? "init" : "");
				sb.Append("\t(\n");

				foreach(var v in method.Body.Variables)
				{
					sb.AppendFormat("\t\t[{0}] {1}\n", v.Index, v.VariableType);
				}

				sb.Append("\t)\n");
			}

			foreach(var instr in method.Body.Instructions)
				sb.AppendFormat("\t{0}\n", instr.ToString());

			sb.Append("}\n");

			return sb.ToString();
		}
		
		[Conditional("DEBUG")]
		public static void DumpMethod( MethodDefinition method )
		{
			foreach(var instr in method.Body.Instructions)
			{
				Console.WriteLine("\t{0} {1}", instr.OpCode.Name, instr.Operand);
			}
		}
		
		[Conditional("DEBUG")]
		public static void DumpTypes( AssemblyDefinition assembly )
		{
			foreach(var t in assembly.MainModule.Types)
			{
				Debug.Print(t.ToString());
			}
		}

		[Conditional("DEBUG")]
		public static void DumpCustomAttributes( AssemblyDefinition assembly )
		{
			foreach(var cat in assembly.CustomAttributes)
			{
				Debug.Print(cat.AttributeType.ToString());
			}
		}
	}
}